@@ -9,9 +9,7 @@
 # or submit itself to any jurisdiction.                                       #
 ###############################################################################
 
-
-import copy
-import json
+import math
 import re
 from dataclasses import dataclass, field
 from itertools import combinations
@@ -33,42 +31,6 @@ class CheckResult:
     tree_data: Dict[str, Dict] = field(default_factory=dict)
 
 
-def checks_to_JSON(
-    checks_data,
-    all_check_results,
-    json_output_path=None,
-):
-    """
-    Serialise information about all checks into a JSON format
-    """
-    all_check_results_copy = copy.deepcopy(all_check_results)
-
-    result = {}
-    for job in all_check_results_copy:
-        result[job] = {}
-        for check in all_check_results_copy[job]:
-            result[job][check] = {}
-
-            result[job][check]["passed"] = all_check_results_copy[job][check].passed
-            result[job][check]["messages"] = all_check_results_copy[job][check].messages
-            result[job][check]["input"] = checks_data[check]
-            result[job][check]["output"] = all_check_results_copy[job][check].tree_data
-
-            # Temporary: remove histograms until a good way to serialise them is found
-            for tree in result[job][check]["output"]:
-                if "histograms" in result[job][check]["output"][tree]:
-                    hist_placeholder = ["placeholder"] * len(
-                        result[job][check]["output"][tree]["histograms"]
-                    )
-                    result[job][check]["output"][tree]["histograms"] = hist_placeholder
-
-    if json_output_path is not None:
-        with open(json_output_path, "w", encoding="utf8") as json_file:
-            json.dump(result, json_file, indent="  ")
-
-    return json.dumps(result, indent="  ")
-
-
 def num_entries(
     filepath_list,
     count,
@@ -190,12 +152,6 @@ def range_check(
                     result.messages += [f"Missing branch in {key!r} with {e!r}"]
                     result.passed = False
                     continue
-                except Exception as e:
-                    result.messages += [
-                        f"Failed to apply {expression!r} to {key!r} with error: {e!r}"
-                    ]
-                    result.passed = False
-                    continue
 
                 # Evaluate expression & apply limits
                 test_array = values_obj[expression]
@@ -314,8 +270,8 @@ def range_check_nd(
     # Check if the number of variables matches expectations
     length_expr = len(expression)
     length_limits = len(limits)
-    if length_expr < 2 or length_expr > 4:
-        result.messages += ["Expected at least two variables, but not more than four."]
+    if length_expr != 2 and length_expr != 3:
+        result.messages += ["Expected two or three variables."]
         result.passed = False
         return result
     if length_expr != length_limits:
@@ -356,6 +312,31 @@ def range_check_nd(
                         )
                         h = Hist(axis0, axis1)
                         result.tree_data[key]["histograms"] += [h]
+                    if length_expr == 3:
+                        for key_i, key_j, key_k in combinations(
+                            list(expression.keys()), 3
+                        ):
+                            axis0 = hist.axis.Regular(
+                                50,
+                                limits[key_i]["min"],
+                                limits[key_i]["max"],
+                                name=expression[key_i],
+                            )
+                            axis1 = hist.axis.Regular(
+                                50,
+                                limits[key_j]["min"],
+                                limits[key_j]["max"],
+                                name=expression[key_j],
+                            )
+                            axis2 = hist.axis.Regular(
+                                50,
+                                limits[key_k]["min"],
+                                limits[key_k]["max"],
+                                name=expression[key_k],
+                            )
+                            h = Hist(axis0, axis1, axis2)
+                        result.tree_data[key]["histograms"] += [h]
+
                     result.tree_data[key]["num_entries"] = 0
 
                 values_obj = {}
@@ -367,12 +348,6 @@ def range_check_nd(
                     result.messages += [f"Missing branch in {key!r} with {e!r}"]
                     result.passed = False
                     continue
-                except Exception as e:
-                    result.messages += [
-                        f"Failed to apply the expressions to {key!r} with error: {e!r}"
-                    ]
-                    result.passed = False
-                    continue
                 # Apply blinding
                 mask_total = []
                 if isinstance(blind_ranges, dict):
@@ -409,6 +384,17 @@ def range_check_nd(
                     h = result.tree_data[key]["histograms"][hist_index]
                     h.fill(values_obj[expression[key_i]], values_obj[expression[key_j]])
                     hist_index += 1
+                # If more than two variables are given in input, return also 3D histograms
+                if length_expr == 3:
+                    for key_i, key_j, key_k in combinations(list_keys, 3):
+                        h = result.tree_data[key]["histograms"][hist_index]
+                        h.fill(
+                            values_obj[expression[key_i]],
+                            values_obj[expression[key_j]],
+                            values_obj[expression[key_k]],
+                        )
+                        hist_index += 1
+
                 # Add to event counter
                 result.tree_data[key]["num_entries"] += values_obj[
                     list_expressions[0]
@@ -430,12 +416,15 @@ def range_check_nd(
                     result.passed = False
                     result.messages += [f"No events found in range for Tree {key}"]
                     continue
-
                 # Histogram check successful
-                result.messages += [
-                    f"Histogram of {expression} successfully filled from TTree {key} (contains {h.sum()} events)"
-                ]
-
+                if len(h.axes) == 2:
+                    message = f"Histogram of {h.axes[0].name}, {h.axes[1].name} successfully filled"
+                    message += f" from TTree {key} (contains {h.sum()} events)"
+                    result.messages += [message]
+                else:
+                    message = f"Histogram of {h.axes[0].name}, {h.axes[1].name}, {h.axes[2].name} successfully filled"
+                    message += f" from TTree {key} (contains {h.sum()} events)"
+                    result.messages += [message]
     return result
 
 
@@ -486,6 +475,9 @@ def num_entries_per_invpb(
                 if re.fullmatch(lumi_pattern, key):
                     try:
                         lumi_arr = obj["IntegratedLuminosity"].array(library="np")
+                        err_lumi_arr = obj["IntegratedLuminosityErr"].array(
+                            library="np"
+                        )
                     except uproot.exceptions.KeyInFileError as e:
                         result.messages += [
                             f"Missing luminosity branch in {key!r} with error {e!r}"
@@ -493,6 +485,13 @@ def num_entries_per_invpb(
                         result.passed = False
                         break
 
+                    err_lumi_quad_sum = math.sqrt(numpy.sum(err_lumi_arr ** 2))
+                    if err_lumi_quad_sum / numpy.sum(lumi_arr) >= 1:
+                        result.passed = False
+                        result.messages += [
+                            "Luminosity information is not reliable: 100% or greater relative uncertainty"
+                        ]
+                        break
                     # Add to luminosity counter
                     lumi += numpy.sum(lumi_arr)
 
@@ -631,12 +630,6 @@ def range_check_bkg_subtracted(
                     result.messages += [f"Missing branch in {key!r} with {e!r}"]
                     result.passed = False
                     continue
-                except Exception as e:
-                    result.messages += [
-                        f"Failed to apply {expr_for_subtraction!r} or {expression!r} to {key!r} with error: {e!r}"
-                    ]
-                    result.passed = False
-                    continue
 
                 # Fill control variable histogram
                 var_for_bkgsub_array = values_obj[expr_for_subtraction]
