@@ -88,7 +88,6 @@ CHECK_TYPE_SCHEMAS = {
                 "x": Str(),
                 "y": Str(),
                 Optional("z"): Str(),
-                Optional("w"): Str(),
             }
         ),
         "limits": Map(
@@ -96,7 +95,6 @@ CHECK_TYPE_SCHEMAS = {
                 "x": Map({"min": Float(), "max": Float()}),
                 "y": Map({"min": Float(), "max": Float()}),
                 Optional("z"): Map({"min": Float(), "max": Float()}),
-                Optional("w"): Map({"min": Float(), "max": Float()}),
             }
         ),
         Optional("blind_ranges"): Seq(
@@ -105,7 +103,6 @@ CHECK_TYPE_SCHEMAS = {
                     "x": Map({"min": Float(), "max": Float()}),
                     "y": Map({"min": Float(), "max": Float()}),
                     Optional("z"): Map({"min": Float(), "max": Float()}),
-                    Optional("w"): Map({"min": Float(), "max": Float()}),
                 }
             )
         ),
