@@ -264,19 +264,21 @@ def test_range_check_nd_passing():
         [
             "root://eospublic.cern.ch//eos/opendata/lhcb/AntimatterMatters2017/data/B2HHH_MagnetDown.root"
         ],
-        {"x": "H1_PZ", "y": "H2_PZ"},
-        {"x": {"min": 0.0, "max": 500000.0}, "y": {"min": 0.0, "max": 500000.0}},
-        [
-            {
-                "x": {"min": 20000.0, "max": 40000.0},
-                "y": {"min": 20000.0, "max": 40000.0},
-            }
-        ],
+        {"x": "H1_PZ", "y": "H2_PZ", "z": "H2_PX"},
+        {
+            "x": {"min": 0.0, "max": 500000.0},
+            "y": {"min": 0.0, "max": 500000.0},
+            "z": {"min": 0.0, "max": 500000.0},
+        },
+        [],
         "DecayTree",
     )
     assert result.passed
     assert result.messages == [
-        "Histogram of {'x': 'H1_PZ', 'y': 'H2_PZ'} successfully filled from TTree DecayTree (contains 4879124.0 events)"
+        "Histogram of H1_PZ, H2_PZ successfully filled from TTree DecayTree (contains 5134453.0 events)",
+        "Histogram of H1_PZ, H2_PX successfully filled from TTree DecayTree (contains 2525193.0 events)",
+        "Histogram of H2_PZ, H2_PX successfully filled from TTree DecayTree (contains 2525806.0 events)",
+        "Histogram of H1_PZ, H2_PZ, H2_PX successfully filled from TTree DecayTree (contains 2525189.0 events)",
     ]
     assert list(result.tree_data.keys()) == ["DecayTree"]
     assert not result.tree_data["DecayTree"]["histograms"] == []
@@ -293,9 +295,7 @@ def test_range_check_nd_failing_num_var():
         "DecayTree",
     )
     assert not result.passed
-    assert result.messages == [
-        "Expected at least two variables, but not more than four."
-    ]
+    assert result.messages == ["Expected two or three variables."]
     assert list(result.tree_data.keys()) == []
 
 
@@ -385,7 +385,7 @@ def test_range_check_bkg_subtracted_passing():
         30.0,
         10.0,
         20.0,
-        [],
+        {"min": 10000.0, "max": 30000.0},
         "DecayTree",
     )
     assert result.passed
